#include"human.hpp"
#include"system.hpp"
#include"place.hpp"


enum{
  motion_number = 6,
};


Human::
Human(ObjectUID  uid, const char16_t*  name, Place*  place, int  x, int  y):
Object(OBJ_human,uid,name),
current_place(nullptr),
current(nullptr)
{
  shpctx.texture = sys.texture;

  reset(place,x,y);

  movctx.clock.set_delay(40);

  shpctx.set_size(tile_size,tile_size*2);

  start_stand();
}




void
Human::
reset(Place*  place, int  x, int  y)
{
    if(current_place != place)
    {
        if(current_place)
        {
          current_place->human_group.eject(get_member_index());
        }


        if(place)
        {
          place->human_group.insert(this);
        }
    }


  current_place = place;

  reset(x,y);
}


void
Human::
reset(int  x, int  y)
{
    if(current_place)
    {
      current = &current_place->get(x,y);

      movctx.point = current->point;

      movctx.dst_point.x = (x*tile_size);
      movctx.dst_point.y = (y*tile_size);

      current->object = this;
    }
}


void
Human::
start_stand()
{
  movctx.motion_type = MOT_stand;
  movctx.motion_phase = 0;
  movctx.motion_count = 0;
}


void
Human::
start_walk(Dir  dir)
{
  PlaceElement*  next = nullptr;

    switch(dir)
    {
      case(DIR_left ): next = current->left ;break;
      case(DIR_right): next = current->right;break;
      case(DIR_back ): next = current->up   ;break;
      case(DIR_front): next = current->down ;break;

      default:
      report;
    }


    if(next)
    {
        if(!next->object)
        {
          movctx.point = next->point;

          current->object = nullptr;

          movctx.motion_type  = MOT_walk;
          movctx.motion_phase =        0;
          movctx.motion_count =        0;

          movctx.motion_dir = dir;

          current = next;

          current->object = this;
        }
    }


  movctx.shape_dir = dir;

    switch(dir)
    {
      case(DIR_left):
        movctx.src_x_base = tile_size*motion_number;
        movctx.src_y_base = 0;

        shpctx.flip = SDL_FLIP_HORIZONTAL;
        break;
      case(DIR_right):
        movctx.src_x_base = tile_size*motion_number;
        movctx.src_y_base = 0;

        shpctx.flip = SDL_FLIP_NONE;
        break;
      case(DIR_back):
        movctx.src_x_base = tile_size*motion_number;
        movctx.src_y_base = tile_size*2*2;

        shpctx.flip = SDL_FLIP_NONE;
        break;
      case(DIR_front):
        movctx.src_x_base = tile_size*motion_number;
        movctx.src_y_base = tile_size*2;

        shpctx.flip = SDL_FLIP_NONE;
        break;
    }


  shpctx.src_rect.x = movctx.src_x_base;
  shpctx.src_rect.y = movctx.src_y_base;

  movctx.motion_count = 0;
}


void
Human::
walk()
{
  enum{v = 4};

    if(movctx.motion_phase >= 8)
    {
      shpctx.src_rect.x = movctx.src_x_base;
      shpctx.src_rect.y = movctx.src_y_base;

      start_stand();
            stand();

        if(movctx.motion_type != MOT_walk)
        {
          return;
        }
    }


    switch(movctx.motion_dir)
    {
      case(DIR_left ): movctx.dst_point.x -= v;break;
      case(DIR_right): movctx.dst_point.x += v;break;
      case(DIR_back ): movctx.dst_point.y -= v;break;
      case(DIR_front): movctx.dst_point.y += v;break;
    }


  shpctx.src_rect.x = movctx.src_x_base-(tile_size*(motion_number-(movctx.motion_count/3)));
  shpctx.src_rect.y = movctx.src_y_base                                             ;

  ++movctx.motion_phase;
  ++movctx.motion_count;
}


void
Human::
move()
{
    switch(movctx.motion_type)
    {
      case(MOT_stand):
        stand();
        break;
      case(MOT_walk):
        walk();
        break;
    }
}


void
Human::
update()
{
    if(movctx.clock())
    {
      move();
    }
}



void
Human::
draw() const
{
  shpctx.draw(movctx.dst_point.x,
              movctx.dst_point.y);
}




