#include<SDL.h>
#include<vector>
#include<cstdint>
#include<cstdio>
#include<list>
#include"webpbuffer.hpp"
#include"human.hpp"
#include"system.hpp"
#include"print.hpp"
#include"text_area.hpp"
#include"button.hpp"




namespace{


SDL_Thread*
th;


SDL_Point
pointing_pos;


void
process_keyup_event(SDL_KeyboardEvent&  key)
{
    switch(key.keysym.sym)
    {
      case(SDLK_RETURN): unset(KEY_p    ,KEY_p_unpressed    );break;
      case(SDLK_LEFT  ): unset(KEY_left ,KEY_left_unpressed );break;
      case(SDLK_RIGHT ): unset(KEY_right,KEY_right_unpressed);break;
      case(SDLK_UP    ): unset(KEY_up   ,KEY_up_unpressed   );break;
      case(SDLK_DOWN  ): unset(KEY_down ,KEY_down_unpressed );break;
    }
}


void
process_keydown_event(SDL_KeyboardEvent&  key)
{
    switch(key.keysym.sym)
    {
      case(SDLK_RETURN): set(KEY_p    ,KEY_p_pressed    );break;
      case(SDLK_LEFT  ): set(KEY_left ,KEY_left_pressed );break;
      case(SDLK_RIGHT ): set(KEY_right,KEY_right_pressed);break;
      case(SDLK_UP    ): set(KEY_up   ,KEY_up_pressed   );break;
      case(SDLK_DOWN  ): set(KEY_down ,KEY_down_pressed );break;
    }
}


void
process_event()
{
  static SDL_Event  evt;

    while(SDL_PollEvent(&evt))
    {
        switch(evt.type)
        {
          case(SDL_KEYDOWN):
              if(!evt.key.repeat)
              {
                process_keydown_event(evt.key);
              }

            break;
          case(SDL_KEYUP):
            process_keyup_event(evt.key);
            break;
          case(SDL_QUIT):
            sys.looping_flag = false;
            sys.streaming_flag = false;

            SDL_WaitThread(th,nullptr);
            break;
        }
    }
}


int
update_world(void*  ptr)
{
    while(sys.looping_flag)
    {
     if(test(KEY_left )){pointing_pos.x-=4;}
else if(test(KEY_right)){pointing_pos.x+=4;}

     if(test(KEY_up  )){pointing_pos.y-=4;}
else if(test(KEY_down)){pointing_pos.y+=4;}


        if(!sys.streaming_flag || sys.text_waiting_flag)
        {
        }

      else
        {
          world.update();
        }


      clear_modified();

      SDL_Delay(20);
    }


  return 0;
}


}




int
main(int  argc, char**  argv)
{
  SDL_Init(SDL_INIT_VIDEO);

  SDL_Window*  win = SDL_CreateWindow("tesp -" __DATE__,SDL_WINDOWPOS_CENTERED,SDL_WINDOWPOS_CENTERED,screen_w,screen_h,0);

  sys.looping_flag   = true;
  sys.streaming_flag = true;

  sys.renderer = SDL_CreateRenderer(win,-1,0);

  sys.texture      = load_texture("tesp.webp",sys.renderer);
  sys.icon_texture = load_texture("icons.webp",sys.renderer);

  th = SDL_CreateThread(update_world,"",nullptr);

  reset_cjk();

  uint32_t  last = 0;

  world.resize(5,5);

  player.shpctx.texture = sys.texture;

  player.reset(&world,4,4);

  new NonPlayer(u"NPC-A",&world,1,2);
  new NonPlayer(u"NPC-B",&world,1,0);
  new NonPlayer(u"NPC-C",&world,0,2);

  Clock  drawing_clock;

    while(sys.looping_flag)
    {
      process_event();

      Clock::update();

        if(sys.text_waiting_flag)
        {
            if(test(KEY_p_unpressed) && !text::get_buffer_remain())
            {
              text::clear();

              sys.text_waiting_flag = false;
            }
        }


        if(drawing_clock())
        {
          sys.draw();

          text::push_from_buffer();

          text::draw();

draw(ICO_pointing,pointing_pos.x,pointing_pos.y);

          SDL_RenderPresent(sys.renderer);
        }


      SDL_Delay(20);
    }


  SDL_DestroyTexture(sys.texture);
  SDL_DestroyRenderer(sys.renderer);
  SDL_DestroyWindow(win);

  SDL_Quit();

  return 0;
}




