#include"print.hpp"
#include"system.hpp"




namespace{


enum
{
  width  = 16,
  height = 16,

};


struct
AsciiGlyph
{
  uint8_t  data[height];

};


AsciiGlyph
const ascii_table[] =
{
#include"ascii.txt"
};


uint16_t
const cjk_source[] =
{
#include"cjk.txt"
};


const uint16_t*
unicode_glyphs[0x10000];


std::vector<SDL_Point>
points;


void
put_ascii(char16_t  c, int  x, int  y)
{
  uint8_t  const*  p = ascii_table[c].data;

    rep(yy,height)
    {
      uint8_t  v = *p++;

        rep(xx,(width/2))
        {
            if(v&0x80)
            {
              SDL_Point  pt = {x+xx,y+yy};

              points.emplace_back(pt);
            }


          v <<= 1;
        }
    }
}


void
put_unicode_internal(char16_t  c, int  x, int  y)
{
  uint16_t  const*  p = unicode_glyphs[c];

    rep(yy,14)
    {
      uint16_t  v = *p++;

        rep(xx,14)
        {
            if(v&0x8000)
            {
              SDL_Point  pt = {x+xx,y+yy};

              points.emplace_back(pt);
            }


          v  <<= 1;
        }
    }
}




}




void
reset_cjk()
{
  static const int  n = (sizeof(cjk_source)/(sizeof(uint16_t)*15));

  auto  p = cjk_source;

    rep(i,n)
    {
      unicode_glyphs[*p] = (p+1);

      p += 15;
    }
}


void
print(const char*  s, int  x, int  y)
{
  points.clear();

    while(*s)
    {
      char16_t  c;

      const size_t  bytes = decode_utf8(s,c);

      x += put_unicode(c,x,y);

      s += bytes;
    }


  SDL_RenderDrawPoints(sys.renderer,points.data(),points.size());
}


void
print(int  x, int  y, const char*  fmt, ...)
{
  va_list  ap;
  va_start(ap,fmt);

  char  sbuf[256];

  vsnprintf(sbuf,sizeof(sbuf),fmt,ap);

  print(sbuf,x,y);

  va_end(ap);
}


int
put_unicode(char16_t  c, int  x, int  y)
{
  int  retval;

    if(c <= 0x7F)
    {
      put_ascii(c,x,y);

      retval = width/2;
    }

  else
    {
      put_unicode_internal(c,x,y);

      retval = width;
    }


  return retval;
}


void
print(const char16_t*  s, int  x, int  y)
{
  points.clear();

    while(*s)
    {
      const char16_t  c = *s++;

      x += put_unicode(c,x,y);
    }


  SDL_RenderDrawPoints(sys.renderer,points.data(),points.size());
}




