#include"util.hpp"




size_t
get_utf8_bytes(const char*  s)
{
  const int  c = *(const unsigned char*)s;

       if(!(c>>7)          ){return 1;}
  else if( (c>>4) == 0b1110){return 3;}
  else if( (c>>5) == 0b110 ){return 2;}
  else if( (c>>6) == 0b10  ){return 1;}


  return 0;
}


size_t
get_utf8_length(const char*  s)
{
  size_t  length = 0;

    for(;;)
    {
      const size_t  bytes = get_utf8_bytes(s);

        if(!bytes)
        {
          break;
        }


      ++length;

      s += bytes;
    }


  return length;
}


size_t
decode_utf8(const char*  s, char16_t&  c)
{
  const unsigned char*  p = (const unsigned char*)s;

  const size_t  bytes = get_utf8_bytes(s);

    switch(bytes)
    {
      case(1): c = ((p[0]             )                                    );break;
      case(2): c = ((p[0]&0b11111)<< 6)|((p[1]&0b111111)                   );break;
      case(3): c = ((p[0]&0b01111)<<12)|((p[1]&0b111111)<<6)|(p[2]&0b111111);break;
    }


  return bytes;
}


size_t
encode_to_utf8(char16_t  c, char*  buf, size_t  buf_size)
{
    if(buf_size <= 3)
    {
      return 0;
    }


  size_t  bytes = 0;

    if(c <= 0x7F)
    {
      *buf++ = c;

      bytes = 1;
    }

  else
    if((c >= 0x0080) &&
       (c <= 0x07FF))
    {
      *buf++ = (0b11000000|(c>>6));
      *buf++ = (0b10000000|(c&0b111111));

      bytes = 2;
    }

  else
    if((c >= 0x0800) &&
       (c <= 0xFFFF))
    {
      *buf++ = (0b11100000|(c>>12));
      *buf++ = (0b10000000|((c>>6)&0b111111));
      *buf++ = (0b10000000|((c   )&0b111111));

      bytes = 3;
    }


  *buf = 0;

  return bytes;
}




